@@ -23,7 +23,6 @@ if TYPE_CHECKING:
 
 LOG = getLogger(__name__)
 
-__all__ = ("ADBCommandError", "ADBCommunicationError", "ADBSession", "ADBSessionError")
 __author__ = "Tyson Smith"
 __credits__ = ["Tyson Smith", "Jesse Schwartzentruber"]
 
@@ -82,37 +81,34 @@ class ADBSession:
 
     Attributes:
         _adb_bin: ADB binary to use.
-        _cpu_arch: Android CPU architecture string.
         _debug_adb: Include ADB output in debug output.
         _ip_addr: Target device IP address.
-        _os_version: Android version string.
         _port: ADB listening port.
         _root: Connected as root user.
         connected: ADB connection state.
+        device_id: Unique Android ID.
         symbols: Location of symbols on the local machine.
     """
 
     __slots__ = (
         "_adb_bin",
-        "_cpu_arch",
         "_debug_adb",
         "_ip_addr",
-        "_os_version",
         "_port",
         "_root",
         "connected",
+        "device_id",
         "symbols",
     )
 
     def __init__(self, ip_addr: str | None = None, port: int = 5555) -> None:
         self._adb_bin = self._adb_check()
-        self._cpu_arch: str | None = None
         self._debug_adb = getenv("SHOW_ADB_DEBUG", "0") != "0"
         self._ip_addr: str | None = None
-        self._os_version: str | None = None
         self._port: int | None = None
         self._root = False
         self.connected = False
+        self.device_id: str | None = None
         self.symbols: dict[str, Path] = {}
 
         if ip_addr is not None:
@@ -175,7 +171,7 @@ class ADBSession:
         return Path(installed_bin)
 
     @staticmethod
-    def _call_adb(cmd: Sequence[str], timeout: int | None = None) -> ADBResult:
+    def _call_adb(cmd: Sequence[str], timeout: int) -> ADBResult:
         """Wrapper to make calls to ADB. Launches ADB in a subprocess and collects
         output. If timeout is specified and elapses the ADB subprocess is terminated.
         This function is only meant to be called directly by ADBSession.call().
@@ -283,7 +279,8 @@ class ADBSession:
         assert args
         if self._debug_adb:
             LOG.debug("call '%s' (%d)", " ".join(args), timeout)
-        if not self.connected and args[0] not in ("connect", "devices", "disconnect"):
+        # a few adb commands do not require a connection
+        if not self.connected and args[0] not in {"connect", "devices", "disconnect"}:
             raise ADBCommunicationError("ADB session is not connected!")
         result = self._call_adb((str(self._adb_bin), *args), timeout=timeout)
         if self._debug_adb:
@@ -297,17 +294,22 @@ class ADBSession:
                 raise ADBCommandError(f"Invalid ADB command '{' '.join(args)}'")
             if result.output.startswith("adb: usage:"):
                 raise ADBCommandError(result.output)
-            if device_required and (
-                result.output.startswith("error: closed")
-                or result.output.startswith("error: device offline")
-                or result.output.startswith("error: no devices/emulators found")
-            ):
-                self.connected = False
-                raise ADBCommunicationError("No device detected!")
+            if device_required:
+                if result.output.startswith("error: device offline"):
+                    LOG.error("ADB call failed: device offline (%s)", self.device_id)
+                    raise ADBCommunicationError("Device offline")
+                if result.output.startswith("error: no devices/emulators found"):
+                    LOG.error("ADB call failed: device not found (%s)", self.device_id)
+                    raise ADBCommunicationError("Device not found")
+                if result.output.startswith("error: closed"):
+                    LOG.error("ADB call failed: device closed (%s)", self.device_id)
+                    raise ADBCommunicationError("Device closed")
+            if result.exit_code != 1:
+                LOG.warning("ADB exit code: %d", result.exit_code)
         return result
 
     def clear_logs(self) -> bool:
-        """Call 'adb logcat --clear' to wipe logs
+        """Call 'adb logcat --clear' to wipe logs.
 
         Args:
             None
@@ -357,55 +359,51 @@ class ADBSession:
         """
         assert boot_timeout > 0
         assert max_attempts > 0
-        assert retry_delay > 0
-        self._cpu_arch = None
-        self._os_version = None
+        assert retry_delay >= 0
         attempt = 0
         root_called = False
         set_enforce_called = False
         while attempt < max_attempts:
             attempt += 1
-            if not self.connected:
-                if self._ip_addr is not None:
-                    addr = f"{self._ip_addr}:{self._port}"
-                    LOG.debug("connecting to '%s'", addr)
-                    if self.call(["connect", addr], timeout=30).exit_code != 0:
-                        LOG.warning("connection attempt #%d failed", attempt)
-                        sleep(retry_delay)
-                        continue
-                elif not self.devices():
-                    LOG.warning(
-                        "No device detected (attempt %d/%d)", attempt, max_attempts
-                    )
+            # attempt to connect to device
+            self.connected = False
+            if self._ip_addr is not None:
+                addr = f"{self._ip_addr}:{self._port}"
+                LOG.debug("connecting to '%s'", addr)
+                if self.call(["connect", addr], timeout=30).exit_code != 0:
+                    LOG.warning("connection attempt #%d failed", attempt)
                     sleep(retry_delay)
                     continue
-                self.connected = True
-            # verify we are connected
-            if not self.wait_for_boot(timeout=boot_timeout):
-                raise ADBCommunicationError(
-                    f"Timeout ({boot_timeout}s) waiting for device to boot"
-                )
-            result = self.call(
-                ["shell", "-T", "-n", "whoami"], device_required=False, timeout=30
-            )
-            if result.exit_code != 0 or not result.output:
-                self.connected = False
-                if attempt == max_attempts:
-                    raise ADBSessionError(
-                        "Device in a bad state, try disconnect & reboot"
-                    )
+            elif not self.devices():
+                LOG.warning("No device detected (attempt %d/%d)", attempt, max_attempts)
+                sleep(retry_delay)
                 continue
-            self._root = result.output.splitlines()[-1] == "root"
-            # collect CPU and OS info
-            if self._os_version is None:
-                self._os_version = self.shell(
-                    ["getprop", "ro.build.version.release"]
-                ).output
-            if self._cpu_arch is None:
-                self._cpu_arch = self.shell(["getprop", "ro.product.cpu.abi"]).output
+            self.connected = True
+            # verify we are connected
+            LOG.debug("waiting for device to boot (%ds)...", boot_timeout)
+            if not self.wait_for_boot(timeout=boot_timeout):
+                LOG.debug("device failed to boot (%ds)", boot_timeout)
+                self.connected = False
+                raise ADBCommunicationError("Device boot timeout exceeded")
+            # collect android id from device
+            # this also helps to ensure the device is functioning properly
+            result = self.shell(
+                ["settings", "get", "secure", "android_id"], device_required=False
+            )
+            if result.exit_code != 0:
+                LOG.error("Failed to retrieve Android ID")
+                raise ADBSessionError("Device in invalid state")
+            self.device_id = result.output
+            # get active user
+            result = self.shell(["whoami"], device_required=False, timeout=30)
+            if result.exit_code != 0 or not result.output:
+                LOG.error("Failed to retrieve active user")
+                raise ADBSessionError("Device in invalid state")
+            self._root = result.output == "root"
             # check SELinux mode
             if self._root and self.get_enforce():
                 if set_enforce_called:
+                    self.connected = False
                     raise ADBSessionError("set_enforce(0) failed!")
                 # set SELinux to run in permissive mode
                 self.set_enforce(0)
@@ -413,31 +411,28 @@ class ADBSession:
                 self.shell(["start"])
                 # put the device in a known state
                 self.call(["reconnect"], timeout=30)
-                self.connected = False
                 set_enforce_called = True
                 attempt -= 1
                 continue
-            if not as_root or self._root:
-                LOG.debug(
-                    "connected device running Android %s (%s)",
-                    self._os_version,
-                    self._cpu_arch,
-                )
-                break
-            # enable root
-            assert as_root, "Should not be here if root is not requested"
-            if self.call(["root"], timeout=30).exit_code == 0:
-                self.connected = False
-                # only skip attempt to call root once
-                if not root_called:
-                    root_called = True
-                    attempt -= 1
-                    continue
-            else:
-                LOG.warning("Failed root login attempt")
-        if self.connected and as_root and not self._root:
-            raise ADBSessionError("Could not enable root")
-        return self.connected
+            # enable root if needed
+            if as_root and not self._root:
+                if self.call(["root"], timeout=30).exit_code == 0:
+                    # only skip attempt to call root once
+                    if not root_called:
+                        root_called = True
+                        attempt -= 1
+                else:
+                    LOG.warning("Failed root login attempt")
+                continue
+            # connected!
+            break
+        else:
+            LOG.debug("failed to connect to device")
+            self.connected = False
+            return False
+
+        assert self.connected
+        return True
 
     @classmethod
     def create(
@@ -839,7 +834,7 @@ class ADBSession:
             cmd.append(f"tcp:{remote}")
         else:
             cmd.append("--remove-all")
-        return self.call(cmd, timeout=10).exit_code == 0
+        return self.call(cmd, device_required=False, timeout=10).exit_code == 0
 
     def sanitizer_options(self, prefix: str, options: Mapping[str, str]) -> None:
         """Set sanitizer options.
@@ -875,18 +870,25 @@ class ADBSession:
             LOG.warning("set_enforce requires root")
         self.shell(["setenforce", str(value)])
 
-    def shell(self, cmd: Sequence[str], timeout: int = 60) -> ADBResult:
+    def shell(
+        self, cmd: Sequence[str], device_required: bool = True, timeout: int = 60
+    ) -> ADBResult:
         """Execute an ADB shell command via a non-interactive shell.
 
         Args:
             cmd: Strings to pass as arguments when calling ADB.
+            device_required: A device must be available.
             timeout: Seconds to wait for ADB call to complete.
 
         Returns:
             The exit code of the ADB call and stderr and stdout.
         """
         assert cmd
-        return self.call(("shell", "-T", "-n", *cmd), timeout=timeout)
+        return self.call(
+            ("shell", "-T", "-n", *cmd),
+            device_required=device_required,
+            timeout=timeout,
+        )
 
     def uninstall(self, package: str) -> bool:
         """Uninstall package from the connected device.
@@ -902,42 +904,22 @@ class ADBSession:
             return False
         return self.call(["uninstall", package], timeout=60).exit_code == 0
 
-    def wait_for_boot(self, timeout: int | None = None) -> bool:
-        """Uninstall package from the connected device.
+    def wait_for_boot(self, timeout: int, poll_wait: int = 1) -> bool:
+        """Wait for device to boot.
 
         Args:
-            timeout: Seconds to wait for device to boot.
+            timeout: Time in seconds to wait for device to boot.
+            poll_wait: Time in seconds between checks.
 
         Returns:
-            True if device booted successfully otherwise False.
+            True if device has booted successfully otherwise False.
         """
-        if timeout is not None:
-            assert timeout > 0
-            deadline = time() + timeout
-        else:
-            deadline = None
-        # first wait for the boot to complete then wait for the boot animation to
-        # complete, this will help ensure the device is in a ready state
-        anim_chk = ["getprop", "init.svc.bootanim"]
-        boot_chk = ["shell", "-T", "-n", "getprop", "sys.boot_completed"]
-        attempts = 0
-        booted = False
+        deadline = time() + timeout
+        cmd = ("getprop", "sys.boot_completed")
         while True:
-            if not booted:
-                booted = self.call(boot_chk, device_required=False).output == "1"
-                attempts += 1
-            # we need to verify that boot is complete before checking the animation is
-            # stopped because the animation can be in the stopped state early in the
-            # boot process
-            if booted and self.shell(anim_chk).output == "stopped":
-                if attempts > 1:
-                    # the device was booting so give it additional time
-                    LOG.debug("device boot was detected")
-                    sleep(5)
+            if self.shell(cmd, device_required=False).output == "1":
                 return True
-            if deadline and time() >= deadline:
-                LOG.debug("wait_for_boot() timeout %r exceeded", timeout)
+            if time() >= deadline:
                 break
-            LOG.debug("waiting for device to boot")
-            sleep(0.5)
+            sleep(poll_wait)
         return False
