@@ -7,7 +7,6 @@
 from __future__ import annotations
 
 from argparse import ArgumentParser
-from contextlib import suppress
 from enum import Enum, auto
 from logging import DEBUG, INFO, basicConfig, getLogger
 from os import environ, getenv
@@ -15,7 +14,7 @@ from pathlib import Path
 from platform import system
 from shutil import copy, rmtree
 from socket import AF_INET, SO_REUSEADDR, SOCK_STREAM, SOL_SOCKET, socket
-from subprocess import DEVNULL, Popen, TimeoutExpired, check_output
+from subprocess import DEVNULL, Popen, check_output, run
 from tempfile import TemporaryDirectory
 from time import perf_counter, sleep
 from urllib.parse import urlparse
@@ -31,8 +30,8 @@ from xml.etree.ElementTree import (
 from fuzzfetch.download import download_url, get_url, iec
 from fuzzfetch.extract import extract_zip
 
-with suppress(ImportError):
-    from xvfbwrapper import Xvfb
+if system() == "Linux":
+    from xvfbwrapper import Xvfb  # pylint: disable=import-error
 
 __author__ = "Jesse Schwartzentruber"
 
@@ -398,7 +397,7 @@ class AndroidEmulator:
         xvfb: bool = False,
         target: str | None = None,
         verbose: bool = False,
-        boot_timeout: int = 120,
+        boot_timeout: int = 300,
     ) -> None:
         """Create an AndroidEmulator object.
 
@@ -517,36 +516,30 @@ class AndroidEmulator:
             None
         """
         assert boot_timeout > 0
+        cmd = (
+            str(PATHS.sdk_root / "platform-tools" / f"adb{EXE_SUFFIX}"),
+            "shell",
+            "getprop",
+            "sys.boot_completed",
+        )
+        LOG.debug("waiting for device to boot...")
         deadline = perf_counter() + boot_timeout
-        # wait for adb port so we can use wait-for-device
-        with socket(AF_INET, SOCK_STREAM) as sock:
-            sock.settimeout(1)
-            while True:
-                with suppress(ConnectionError, TimeoutError):
-                    sock.connect(("127.0.0.1", port))
-                    break
-                if proc.poll() is not None:
-                    raise AndroidEmulatorError("Incomplete emulator launch.")
-                if deadline <= perf_counter():
-                    raise AndroidEmulatorError("Timeout waiting for ADB port.")
-                sleep(1)
-        # use adb wait-for-device to wait for device to boot
-        try:
-            check_output(
-                [
-                    str(PATHS.sdk_root / "platform-tools" / f"adb{EXE_SUFFIX}"),
-                    "wait-for-device",
-                    "shell",
-                    "while [[ -z $(getprop sys.boot_completed) ]];do sleep 1;done",
-                ],
-                timeout=boot_timeout,
+        while True:
+            adb_result = run(
+                cmd,
                 env={"ANDROID_SERIAL": f"emulator-{port:d}"},
+                capture_output=True,
+                check=False,
+                timeout=30,
             )
-        except TimeoutExpired:
-            raise AndroidEmulatorError("Emulator failed to boot in time.") from None
-
-        if proc.poll() is not None:
-            raise AndroidEmulatorError("Failed to launch emulator.")
+            if adb_result.returncode == 0 and adb_result.stdout.strip() == b"1":
+                LOG.debug("device booted")
+                break
+            if proc.poll() is not None:
+                raise AndroidEmulatorError("Failed to launch emulator.")
+            if perf_counter() >= deadline:
+                raise AndroidEmulatorError("Emulator failed to boot in time.")
+            sleep(1)
 
     def relaunch(self) -> AndroidEmulator:
         """Create a new AndroidEmulator object created with the same parameters used to
@@ -681,7 +674,7 @@ class AndroidEmulator:
         )
 
     def shutdown(self) -> None:
-        """Use the emulator control channel to request clean shutdown.
+        """Close the emulator process. This methods exists for compatibility.
 
         Args:
             None
@@ -690,24 +683,8 @@ class AndroidEmulator:
             None
         """
         LOG.info("Initiating emulator shutdown")
-        with suppress(ConnectionResetError), socket(AF_INET, SOCK_STREAM) as sock:
-            sock.connect(("127.0.0.1", self.port))
-            # read connection message
-            lines = sock.recv(65536).splitlines()
-            try:
-                auth_token_idx = lines.index(
-                    b"Android Console: you can find your <auth_token> in "
-                )
-            except ValueError:
-                pass
-            else:
-                auth_token_path = lines[auth_token_idx + 1].strip(b"'")
-                with open(auth_token_path, "rb") as auth_token_fp:
-                    auth_token = auth_token_fp.read()
-                sock.sendall(b"auth " + auth_token + b"\n")
-                # receive auth message
-                _ = sock.recv(65536)
-            sock.sendall(b"kill\n")
+        # terminate is handled and the emulator attempts a clean shutdown
+        self.terminate()
 
     @staticmethod
     def search_free_ports(search_port: int | None = None) -> int:
@@ -857,8 +834,8 @@ def main(argv: list[str] | None = None) -> None:
     aparser.add_argument(
         "--boot-timeout",
         "-t",
-        default=60,
-        type=float,
+        default=300,
+        type=int,
         help=(
             "Time to wait for Android to boot before retrying emulator launch "
             "(default: %(default)ss)"
@@ -868,9 +845,12 @@ def main(argv: list[str] | None = None) -> None:
     disp_group.add_argument(
         "--headless", action="store_true", help="Run emulator in headless mode"
     )
-    disp_group.add_argument(
-        "--xvfb", action="store_true", help="Run emulator with Xvfb"
-    )
+    if system() == "Linux":
+        disp_group.add_argument(
+            "--xvfb", action="store_true", help="Run emulator with Xvfb"
+        )
+    else:
+        disp_group.set_defaults(xvfb=False)
     aparser.add_argument(
         "--skip-dl",
         "-s",
@@ -887,8 +867,6 @@ def main(argv: list[str] | None = None) -> None:
 
     if args.boot_timeout < 0:
         aparser.error("--boot-timeout must be positive")
-    if args.boot_timeout == 0:
-        args.boot_timeout = None
 
     init_logging(debug=args.verbose)
 
@@ -915,8 +893,6 @@ def main(argv: list[str] | None = None) -> None:
             LOG.info("Android emulator is running on port %d", port)
             try:
                 emu.wait()
-            except KeyboardInterrupt:
-                LOG.info("Aborting...")
             finally:
                 if emu.poll() is None:
                     emu.shutdown()
@@ -925,6 +901,8 @@ def main(argv: list[str] | None = None) -> None:
                     emu.wait(timeout=120)
                 finally:
                     emu.cleanup()
+        except KeyboardInterrupt:
+            LOG.info("Aborting...")
         finally:
             AndroidEmulator.remove_avd(avd_name)
 
